mix yog.test_livebooks (YogEx v1.0.0)

Copy Markdown View Source

Finds and runs all Elixir code blocks inside the .livemd files under the livebooks/ directory. Ensures that the notebooks execute successfully without any runtime compilation or execution errors.

Run this task via:

mix yog.test_livebooks